from datetime import datetime

import pytest
from dateutil.tz import tzutc

from mythx_models.util import (
    deserialize_api_timestamp,
    dict_delete_none_fields,
    serialize_api_timestamp,
)


@pytest.mark.parametrize(
    "api_ts,datetime_ts",
    [
        (
            "2018-11-20T23:13:12.177Z",
            datetime(
                year=2018,
                month=11,
                day=20,
                hour=23,
                minute=13,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2019-11-20T23:13:12.177Z",
            datetime(
                year=2019,
                month=11,
                day=20,
                hour=23,
                minute=13,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-12-20T23:13:12.177Z",
            datetime(
                year=2018,
                month=12,
                day=20,
                hour=23,
                minute=13,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-11-21T23:13:12.177Z",
            datetime(
                year=2018,
                month=11,
                day=21,
                hour=23,
                minute=13,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-11-20T22:13:12.177Z",
            datetime(
                year=2018,
                month=11,
                day=20,
                hour=22,
                minute=13,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-11-20T23:12:12.177Z",
            datetime(
                year=2018,
                month=11,
                day=20,
                hour=23,
                minute=12,
                second=12,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-11-20T23:13:11.177Z",
            datetime(
                year=2018,
                month=11,
                day=20,
                hour=23,
                minute=13,
                second=11,
                microsecond=177000,
                tzinfo=tzutc(),
            ),
        ),
        (
            "2018-11-20T23:13:12.176Z",
            datetime(
                year=2018,
                month=11,
                day=20,
                hour=23,
                minute=13,
                second=12,
                microsecond=176000,
                tzinfo=tzutc(),
            ),
        ),
    ],
)
def test_ts_serde(api_ts, datetime_ts):
    assert deserialize_api_timestamp(api_ts) == datetime_ts
    assert serialize_api_timestamp(datetime_ts) == api_ts


@pytest.mark.parametrize(
    "input_dict,expected",
    [
        ({"test": None}, {}),
        ({"test": "foo"}, {"test": "foo"}),
        ({"test": "foo", "bar": "baz"}, {"test": "foo", "bar": "baz"}),
        ({"test": "foo", "bar": None}, {"test": "foo"}),
        (
            {"test": "foo", "bar": {"baz": "qux"}},
            {"test": "foo", "bar": {"baz": "qux"}},
        ),
        ({"test": "foo", "bar": {"baz": None}}, {"test": "foo", "bar": {}}),
    ],
)
def test_dict_delete_none_fields(input_dict, expected):
    assert dict_delete_none_fields(input_dict) == expected



"""This module contains domain models regrading found issues."""

from enum import Enum
from typing import Any, Dict, List

from mythx_models.base import JSONSerializable


class Severity(str, Enum):
    """An Enum holding the possible severities an issue can have."""

    UNKNOWN = "Unknown"
    NONE = "None"
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"


class SourceType(str, Enum):
    """An Enum holding the possible source type values."""

    RAW_BYTECODE = "raw-bytecode"
    ETHEREUM_ADDRESS = "ethereum-address"
    SOLIDITY_CONTRACT = "solidity-contract"
    SOLIDITY_FILE = "solidity-file"


class SourceFormat(str, Enum):
    """An Enum holding the possible source format values."""

    TEXT = "text"
    SOLC_AST_LEGACY_JSON = "solc-ast-legacy-json"
    SOLC_AST_COMPACT_JSON = "solc-ast-compact-json"
    EVM_BYZANTIUM_BYTECODE = "evm-byzantium-bytecode"
    EWASM_RAW = "ewasm-raw"


class SourceLocation(JSONSerializable):
    """The domain model for a source location in a detected issue."""

    def __init__(
        self,
        source_map: str,
        source_type: SourceType,
        source_format: SourceFormat,
        source_list: List[str],
    ):
        self.source_map = source_map
        self.source_type = source_type
        self.source_format = source_format
        self.source_list = source_list

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        return cls(
            source_map=d["sourceMap"],
            source_type=SourceType(d["sourceType"]),
            source_format=SourceFormat(d["sourceFormat"]),
            source_list=d["sourceList"],
        )

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        return {
            "sourceMap": self.source_map,
            "sourceType": self.source_type,
            "sourceFormat": self.source_format,
            "sourceList": self.source_list,
        }


class Issue(JSONSerializable):
    """The API response domain model for a single issue object."""

    def __init__(
        self,
        swc_id: str,
        swc_title: str,
        description_short: str,
        description_long: str,
        severity: Severity,
        locations: List[SourceLocation],
        extra: Dict[str, Any],
    ):
        self.swc_id = swc_id
        self.swc_title = swc_title
        self.description_short = description_short
        self.description_long = description_long
        self.severity = severity
        self.locations = locations
        self.extra_data = extra

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        locs = [
            SourceLocation(
                source_map=loc.get("sourceMap"),
                source_type=loc.get("sourceType"),
                source_format=loc.get("sourceFormat"),
                source_list=loc.get("sourceList"),
            )
            for loc in d["locations"]
        ]
        return cls(
            swc_id=d["swcID"],
            swc_title=d["swcTitle"],
            description_short=d["description"]["head"],
            description_long=d["description"]["tail"],
            severity=Severity(d["severity"]) if d["severity"] else Severity.NONE,
            locations=locs,
            extra=d["extra"],
        )

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        return {
            "swcID": self.swc_id,
            "swcTitle": self.swc_title,
            "description": {
                "head": self.description_short,
                "tail": self.description_long,
            },
            "severity": self.severity,
            "locations": [loc.to_dict() for loc in self.locations],
            "extra": self.extra_data,
        }



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AuthRefreshRequest

from . import common as testdata


def assert_auth_refresh_request(req: AuthRefreshRequest):
    assert req.access_token == testdata.ACCESS_TOKEN_1
    assert req.refresh_token == testdata.REFRESH_TOKEN_1
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == testdata.REFRESH_REQUEST_PAYLOAD_DICT


def test_auth_refresh_request_from_valid_json():
    resp = AuthRefreshRequest.from_json(json.dumps(testdata.REFRESH_REQUEST_DICT))
    assert_auth_refresh_request(resp)


def test_auth_refresh_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthRefreshRequest.from_json("{}")


def test_auth_refresh_request_from_valid_dict():
    resp = AuthRefreshRequest.from_dict(testdata.REFRESH_REQUEST_DICT)
    assert_auth_refresh_request(resp)


def test_auth_refresh_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthRefreshRequest.from_dict({})


def test_auth_refresh_request_to_json():
    assert (
        json.loads(testdata.REFRESH_REQUEST_OBJECT.to_json())
        == testdata.REFRESH_REQUEST_DICT
    )


def test_auth_refresh_request_to_dict():
    assert testdata.REFRESH_REQUEST_OBJECT.to_dict() == testdata.REFRESH_REQUEST_DICT



"""This module contains the base domain model."""

import abc
import json
import logging

import jsonschema

from mythx_models.exceptions import ValidationError

LOGGER = logging.getLogger(__name__)


class JSONSerializable(abc.ABC):
    """An abstract base class defining an interface for a JSON serializable class."""

    @classmethod
    def from_json(cls, json_str: str):
        """Deserialize a given JSON string to the given domain model.
        Internally, this method uses the :code:`from_dict` method.

        :param json_str: The JSON string to deserialize
        :return: The concrete deserialized domain model instance
        """
        try:
            parsed = json.loads(json_str)
        except json.JSONDecodeError as exc:
            raise ValidationError(exc)
        return cls.from_dict(parsed)

    def to_json(self):
        """Serialize the current domain model instance to a JSON string.
        Internally, this method uses the :code:`to_dict` method.

        :return: The serialized domain model JSON string
        """
        return json.dumps(self.to_dict())

    @classmethod
    @abc.abstractmethod
    def from_dict(cls, d: dict):
        """An abstract method to construct the given domain model from a Python dict instance.

        :param d: The dict instance to deserialize
        """
        pass  # pragma: no cover

    @abc.abstractmethod
    def to_dict(self):
        """An abstract method to serialize the current domain model instance to a Python dict.

        :return: A Python dict instance holding the serialized domain model data
        """
        pass  # pragma: no cover


class BaseModel(JSONSerializable, abc.ABC):
    """An abstract object describing responses from the MythX API."""

    schema = None

    @classmethod
    def validate(cls, candidate):
        """Validate the object's data format.
        This is done using a schema contained at the class level. If no schema is given, it is
        assumed that the request does not contain any meaningful data (e.g. an empty logout
        response) and no validation is done.
        If the schema validation fails, a :code:`ValidationError` is raised.
        If this method is called on a concrete object that does not contain a schema,
        :code:`validate` will return right away and log a warning as this behaviour might not have
        been intended by a developer.

        :param candidate: The candidate dict to check the schema against
        :return: None
        """
        if cls.schema is None:
            LOGGER.warning("Cannot validate {} without a schema".format(cls.__name__))
            return
        try:
            jsonschema.validate(candidate, cls.schema)
        except jsonschema.ValidationError as e:
            raise ValidationError(e)



from mythx_models.response.analysis_submission import AnalysisSubmissionResponse


class AnalysisStatusResponse(AnalysisSubmissionResponse):
    """The API response domain model for the status of a single analysis."""

    pass



import json

from mythx_models.response import (
    Issue,
    Severity,
    SourceFormat,
    SourceLocation,
    SourceType,
)

from . import common as testdata


def assert_issue(issue: Issue):
    assert issue.swc_id == testdata.SWC_ID
    assert issue.swc_title == testdata.SWC_TITLE
    assert issue.description_short == testdata.DESCRIPTION_HEAD
    assert issue.description_long == testdata.DESCRIPTION_TAIL
    assert issue.severity == Severity(testdata.SEVERITY)
    assert len(issue.locations) == 1
    location = issue.locations[0]
    assert location.source_map == testdata.SOURCE_MAP
    assert location.source_format == SourceFormat.EVM_BYZANTIUM_BYTECODE
    assert location.source_type == SourceType.RAW_BYTECODE
    assert location.source_list == testdata.SOURCE_LIST


def test_issue_from_valid_json():
    issue = Issue.from_json(json.dumps(testdata.ISSUE_DICT))
    assert_issue(issue)


def test_issue_from_dict():
    issue = Issue.from_dict(testdata.ISSUE_DICT)
    assert_issue(issue)


def test_issue_to_json():
    assert json.loads(testdata.ISSUE_OBJECT.to_json()) == testdata.ISSUE_DICT


def test_issue_to_dict():
    assert testdata.ISSUE_OBJECT.to_dict() == testdata.ISSUE_DICT


def test_source_location_from_dict():
    sl = SourceLocation.from_dict(testdata.SOURCE_LOCATION)
    assert sl.source_format == testdata.SOURCE_FORMAT
    assert sl.source_list == testdata.SOURCE_LIST
    assert sl.source_map == testdata.SOURCE_MAP
    assert sl.source_type == testdata.SOURCE_TYPE



"""This module contains domain models regarding analysis jobs"""

from enum import Enum

from inflection import underscore

from mythx_models.response.base import BaseResponse
from mythx_models.util import deserialize_api_timestamp, serialize_api_timestamp


class AnalysisStatus(str, Enum):
    """An Enum describing the status an analysis job can be in."""

    QUEUED = "Queued"
    IN_PROGRESS = "In Progress"
    ERROR = "Error"
    FINISHED = "Finished"


class Analysis(BaseResponse):
    """An object describing an analysis job.

    Such a model was built, because many other API responses deliver the same data when it comes
    to analysis jobs. This makes the code more DRY, validation easier, and allows for recursive
    SerDe (e.g. mapping :code:`from_dict` to a deserialized JSON list of job objects.
    """

    def __init__(
        self,
        uuid: str,
        api_version: str,
        mythril_version: str,
        harvey_version: str,
        maru_version: str,
        queue_time: int,
        status: AnalysisStatus,
        submitted_at: str,
        submitted_by: str,
        run_time: int = 0,
        client_tool_name: str = None,
        error: str = None,
    ):
        self.uuid = uuid
        self.api_version = api_version
        self.mythril_version = mythril_version
        self.harvey_version = harvey_version
        self.maru_version = maru_version
        self.queue_time = queue_time
        self.run_time = run_time
        self.status = AnalysisStatus(status.title())
        self.submitted_at = deserialize_api_timestamp(submitted_at)
        self.submitted_by = submitted_by
        self.client_tool_name = client_tool_name
        self.error = error

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        d = {underscore(k): v for k, v in d.items()}
        return cls(**d)

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "uuid": self.uuid,
            "apiVersion": self.api_version,
            "mythrilVersion": self.mythril_version,
            "harveyVersion": self.harvey_version,
            "maruVersion": self.maru_version,
            "queueTime": self.queue_time,
            "runTime": self.run_time,
            "status": self.status.title(),
            "submittedAt": serialize_api_timestamp(self.submitted_at),
            "submittedBy": self.submitted_by,
            "clientToolName": self.client_tool_name,
        }
        if self.error is not None:
            d.update({"error": self.error})

        return d

    def __eq__(self, candidate):
        return all(
            (
                self.uuid == candidate.uuid,
                self.api_version == candidate.api_version,
                self.mythril_version == candidate.mythril_version,
                self.harvey_version == candidate.harvey_version,
                self.maru_version == candidate.maru_version,
                self.queue_time == candidate.queue_time,
                self.run_time == candidate.run_time,
                self.status == candidate.status,
                self.submitted_at == candidate.submitted_at,
                self.submitted_by == candidate.submitted_by,
                self.client_tool_name == candidate.client_tool_name,
                self.error == candidate.error,
            )
        )

    def __repr__(self):
        return "<Analysis uuid={} status={}>".format(self.uuid, self.status)



import json
from typing import List

from mythx_models.exceptions import ValidationError
from mythx_models.response.analysis import Analysis
from mythx_models.response.base import BaseResponse
from mythx_models.util import resolve_schema

INDEX_ERROR_MSG = "Analysis at index {} was not fetched"


class AnalysisListResponse(BaseResponse):
    """The API response domain model for a list of analyses."""

    with open(resolve_schema(__file__, "analysis-list.json")) as sf:
        schema = json.load(sf)

    def __init__(self, analyses: List[Analysis], total: int) -> None:
        self.analyses = analyses
        self.total = total

    @classmethod
    def validate(cls, candidate):
        """Validate the response data structure and add an explicit type check.

        :param candidate: The Python dict to validate
        """
        super().validate(candidate)
        if not type(candidate) == dict:
            raise ValidationError(
                "Expected type dict but got {}".format(type(candidate))
            )

    @classmethod
    def from_dict(cls, d: dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        analyses = [Analysis.from_dict(a) for a in d["analyses"]]
        return cls(analyses=analyses, total=d["total"])

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "analyses": [a.to_dict() for a in self.analyses],
            "total": len(self.analyses),
        }
        self.validate(d)
        return d

    def __iter__(self):
        for analysis in self.analyses:
            yield analysis

    def __getitem__(self, idx):
        try:
            return self.analyses[idx]
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __setitem__(self, idx, value):
        try:
            self.analyses[idx] = value
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __delitem__(self, idx):
        try:
            del self.analyses[idx]
            self.total -= 1
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __len__(self):
        return self.total

    def __reversed__(self):
        return reversed(self.analyses)

    def __contains__(self, item):
        if not type(item) in (Analysis, str):
            raise ValueError(
                "Expected type Analysis or str but got {}".format(type(item))
            )
        uuid = item.uuid if type(item) == Analysis else item
        return uuid in map(lambda x: x.uuid, self.analyses)

    def __eq__(self, candidate):
        return self.total == candidate.total and self.analyses == candidate.analyses



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AnalysisListRequest

from . import common as testdata


def assert_analysis_list_request(req):
    assert req.offset == testdata.OFFSET
    assert req.date_from.isoformat() == testdata.DATE_FROM
    assert req.date_to.isoformat() == testdata.DATE_TO
    assert req.payload == {}
    assert req.method == "GET"
    assert req.parameters == testdata.ANALYSIS_LIST_REQUEST_DICT
    assert req.headers == {}


def test_analysis_list_request_from_valid_json():
    req = AnalysisListRequest.from_json(json.dumps(testdata.ANALYSIS_LIST_REQUEST_DICT))
    assert_analysis_list_request(req)


def test_analysis_list_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisListRequest.from_json("{}")


def test_analysis_list_request_from_valid_dict():
    req = AnalysisListRequest.from_dict(testdata.ANALYSIS_LIST_REQUEST_DICT)
    assert_analysis_list_request(req)


def test_analysis_list_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisListRequest.from_dict({})


def test_analysis_list_request_to_json():
    assert (
        json.loads(testdata.ANALYSIS_LIST_REQUEST_OBJECT.to_json())
        == testdata.ANALYSIS_LIST_REQUEST_DICT
    )


def test_analysis_list_request_to_dict():
    assert (
        testdata.ANALYSIS_LIST_REQUEST_OBJECT.to_dict()
        == testdata.ANALYSIS_LIST_REQUEST_DICT
    )



from mythx_models.request import VersionRequest

from . import common as testdata


def assert_version_request(req):
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}
    assert req.endpoint == "v1/version"


def test_auth_logout_request_from_valid_json():
    req = VersionRequest.from_json("{}")
    assert_version_request(req)


def test_auth_logout_request_from_valid_dict():
    req = VersionRequest.from_dict({})
    assert_version_request(req)


def test_auth_logout_request_to_json():
    assert testdata.VERSION_REQUEST_OBJECT.to_json() == "{}"


def test_auth_logout_request_to_dict():
    assert testdata.VERSION_REQUEST_OBJECT.to_dict() == {}



"""This module contains the MythX request domain models"""

from mythx_models.request.analysis_list import AnalysisListRequest
from mythx_models.request.analysis_status import AnalysisStatusRequest
from mythx_models.request.analysis_submission import AnalysisSubmissionRequest
from mythx_models.request.auth_login import AuthLoginRequest
from mythx_models.request.auth_logout import AuthLogoutRequest
from mythx_models.request.auth_refresh import AuthRefreshRequest
from mythx_models.request.detected_issues import DetectedIssuesRequest
from mythx_models.request.oas import OASRequest
from mythx_models.request.version import VersionRequest



"""This module contains the AuthLogoutRequest domain model."""

import json
from typing import Dict

from mythx_models.request.base import BaseRequest
from mythx_models.util import resolve_schema


class AuthLogoutRequest(BaseRequest):
    """Perform an API request that logs out the current user."""

    with open(resolve_schema(__file__, "auth-logout.json")) as sf:
        schema = json.load(sf)

    def __init__(self, global_: bool = False):
        self.global_ = global_

    @property
    def endpoint(self):
        """The API's logout endpoint.

        :return: A string denoting the logout endpoint without the host prefix
        """
        return "v1/auth/logout"

    @property
    def method(self):
        """The HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        return "POST"

    @property
    def parameters(self):
        """Additional URL parameters

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        return {}

    @property
    def headers(self):
        """Additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        return {}

    @property
    def payload(self):
        """The request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        return {}

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the request domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(global_=d["global"])

    def to_dict(self):
        """Serialize the request model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"global": self.global_}
        self.validate(d)
        return d



"""This module contains the MythX response domain models"""

from mythx_models.response.analysis import Analysis, AnalysisStatus
from mythx_models.response.analysis_list import AnalysisListResponse
from mythx_models.response.analysis_status import AnalysisStatusResponse
from mythx_models.response.analysis_submission import AnalysisSubmissionResponse
from mythx_models.response.auth_login import AuthLoginResponse
from mythx_models.response.auth_logout import AuthLogoutResponse
from mythx_models.response.auth_refresh import AuthRefreshResponse
from mythx_models.response.detected_issues import DetectedIssuesResponse, IssueReport
from mythx_models.response.issue import (
    Issue,
    Severity,
    SourceFormat,
    SourceLocation,
    SourceType,
)
from mythx_models.response.oas import OASResponse
from mythx_models.response.version import VersionResponse



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AuthLoginRequest

from . import common as testdata


def assert_login(req: AuthLoginRequest):
    assert req.eth_address == testdata.ETH_ADDRESS
    assert req.password == testdata.PASSWORD
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == testdata.LOGIN_REQUEST_DICT


def test_login_from_valid_json():
    req = AuthLoginRequest.from_json(json.dumps(testdata.LOGIN_REQUEST_DICT))
    assert_login(req)


def test_login_from_invalid_json():
    with pytest.raises(ValidationError):
        AuthLoginRequest.from_json("{}")


def test_login_from_valid_dict():
    req = AuthLoginRequest.from_dict(testdata.LOGIN_REQUEST_DICT)
    assert_login(req)


def test_login_from_invalid_dict():
    with pytest.raises(ValidationError):
        AuthLoginRequest.from_dict({})


def test_login_to_json():
    assert (
        json.loads(testdata.LOGIN_REQUEST_OBJECT.to_json())
        == testdata.LOGIN_REQUEST_DICT
    )


def test_login_to_dict():
    assert testdata.LOGIN_REQUEST_OBJECT.to_dict() == testdata.LOGIN_REQUEST_DICT



#!/usr/bin/env python3

"""The setup script."""

from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

with open("requirements.txt", "r") as f:
    requirements = list(map(str.strip, f.read().split("\n")))[:-1]

setup_requirements = ["pytest-runner"]

test_requirements = ["pytest"]

setup(
    author="Dominik Muhs",
    author_email="dominik.muhs@consensys.net",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="Python domain model classes for the MythX platform",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="mythx",
    name="mythx-models",
    packages=find_packages(),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/dmuhs/mythx-models",
    version="1.1.0",
    zip_safe=False,
)



import json
from copy import deepcopy

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import (
    DetectedIssuesResponse,
    IssueReport,
    Severity,
    SourceFormat,
    SourceType,
)

from . import common as testdata


def assert_detected_issues(resp):
    assert len(resp.issue_reports) == 1
    report = resp.issue_reports[0]
    assert type(report) == IssueReport
    issue = report.issues[0]
    assert issue.swc_id == testdata.SWC_ID
    assert issue.swc_title == testdata.SWC_TITLE
    assert issue.description_short == testdata.DESCRIPTION_HEAD
    assert issue.description_long == testdata.DESCRIPTION_TAIL
    assert issue.severity == Severity(testdata.SEVERITY)
    assert len(issue.locations) == 1
    location = issue.locations[0]
    assert location.source_map == testdata.SOURCE_MAP
    assert location.source_format == SourceFormat.EVM_BYZANTIUM_BYTECODE
    assert location.source_type == SourceType.RAW_BYTECODE
    assert location.source_list == testdata.SOURCE_LIST


def test_detected_issues_from_valid_json():
    resp = DetectedIssuesResponse.from_json(
        # DetectedIssuesResponse,
        json.dumps(testdata.DETECTED_ISSUES_RESPONSE_DICT)
    )
    assert_detected_issues(resp)


def test_detected_issues_from_empty_json():
    resp = DetectedIssuesResponse.from_json("[]")
    assert resp.issue_reports == []


def test_detected_issues_from_dict():
    resp = DetectedIssuesResponse.from_dict(testdata.DETECTED_ISSUES_RESPONSE_DICT)
    assert_detected_issues(resp)


def test_detected_issues_from_list():
    resp = DetectedIssuesResponse.from_dict([testdata.ISSUE_REPORT_DICT])
    assert_detected_issues(resp)


def test_detected_issues_from_invalid_type():
    with pytest.raises(ValidationError):
        DetectedIssuesResponse.from_dict("foo")


def test_detected_issues_from_empty_list():
    resp = DetectedIssuesResponse.from_dict([])
    assert resp.issue_reports == []


def test_detected_issues_from_invalid_dict():
    with pytest.raises(ValidationError):
        DetectedIssuesResponse.from_dict({})


def test_detected_issues_to_json():
    json_str = testdata.DETECTED_ISSUES_RESPONSE_OBJECT.to_json()
    assert (
        json.loads(json_str) == testdata.DETECTED_ISSUES_RESPONSE_DICT["issueReports"]
    )


def test_detected_issues_to_dict():
    resp = testdata.DETECTED_ISSUES_RESPONSE_OBJECT.to_dict()
    assert testdata.DETECTED_ISSUES_RESPONSE_DICT == resp


def test_valid_swc_id_contains():
    assert testdata.SWC_ID in testdata.DETECTED_ISSUES_RESPONSE_OBJECT


def test_valid_swc_id_not_contains():
    assert not "SWC-104" in testdata.DETECTED_ISSUES_RESPONSE_OBJECT


def test_invalid_key_contains():
    with pytest.raises(ValueError):
        1337 in testdata.DETECTED_ISSUES_RESPONSE_OBJECT


def test_response_length():
    resp = deepcopy(testdata.DETECTED_ISSUES_RESPONSE_OBJECT)
    total_report_issues = len(resp)
    assert len(resp) == total_report_issues
    resp.issue_reports.append(
        IssueReport(
            issues=["foo", "bar"],
            source_type=SourceType.RAW_BYTECODE,
            source_format=SourceFormat.EVM_BYZANTIUM_BYTECODE,
            source_list="foo.sol",
            meta_data={},
        )
    )
    assert len(resp) == total_report_issues + 2


def test_issue_iterator():
    for i, report in enumerate(testdata.DETECTED_ISSUES_RESPONSE_OBJECT.issue_reports):
        assert testdata.DETECTED_ISSUES_RESPONSE_OBJECT.issue_reports[i] == report


def test_report_iterator():
    report = testdata.DETECTED_ISSUES_RESPONSE_OBJECT
    for i, issue in enumerate(report):
        assert issue == report.issue_reports[0][i]


def test_issue_valid_getitem():
    assert (
        testdata.DETECTED_ISSUES_RESPONSE_OBJECT.issue_reports[0]
        == testdata.DETECTED_ISSUES_RESPONSE_OBJECT[0]
    )


def test_report_getitem():
    report = testdata.DETECTED_ISSUES_RESPONSE_OBJECT
    for i, issue in enumerate(report):
        assert issue == report.issue_reports[0][i]


def test_invalid_getitem():
    with pytest.raises(IndexError):
        test = testdata.DETECTED_ISSUES_RESPONSE_OBJECT[1337]


def test_valid_setitem():
    resp = deepcopy(testdata.DETECTED_ISSUES_RESPONSE_OBJECT)
    resp[0] = "foo"
    assert resp.issue_reports[0] == "foo"


def test_report_valid_setitem():
    resp = deepcopy(testdata.DETECTED_ISSUES_RESPONSE_OBJECT)
    resp.issue_reports[0][0] = "foo"
    assert resp.issue_reports[0][0] == "foo"


def test_invalid_setitem():
    with pytest.raises(TypeError):
        # string key on list access
        testdata.DETECTED_ISSUES_RESPONSE_OBJECT.issue_reports["foo"] = "bar"


def test_report_invalid_setitem():
    with pytest.raises(TypeError):
        # string key on list access
        testdata.DETECTED_ISSUES_RESPONSE_OBJECT.issue_reports[0]["foo"] = "bar"


def test_valid_delete():
    resp = deepcopy(testdata.DETECTED_ISSUES_RESPONSE_OBJECT)
    del resp[0]
    assert resp.issue_reports == []


def test_valid_report_delete():
    resp = deepcopy(testdata.DETECTED_ISSUES_RESPONSE_OBJECT)
    del resp[0][0]
    assert resp.issue_reports[0].issues == []


def test_invalid_delete():
    with pytest.raises(IndexError):
        del testdata.DETECTED_ISSUES_RESPONSE_OBJECT[1337]


def test_invalid_report_delete():
    with pytest.raises(IndexError):
        del testdata.DETECTED_ISSUES_RESPONSE_OBJECT[0][100]



import json

from mythx_models.response.analysis import Analysis
from mythx_models.response.base import BaseResponse
from mythx_models.util import resolve_schema


class AnalysisSubmissionResponse(BaseResponse):
    """The API response domain model for a successful analysis job submision."""

    with open(resolve_schema(__file__, "analysis-submission.json")) as sf:
        schema = json.load(sf)

    def __init__(self, analysis: Analysis):
        self.analysis = analysis

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        return cls(analysis=Analysis.from_dict(d))

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = self.analysis.to_dict()
        self.validate(d)
        return d

    def __getattr__(self, name):
        return getattr(self.analysis, name)
